#include <sys/stat.h>
#include <AL/alc.h>
#include "SDL.h"
#include "SDL_thread.h"

#include "main.h"
#include "log.h"
#include "pack.h"
#include "music.h"
#include "sound.h"

// This isn't always set?
#ifndef AL_FORMAT_VORBIS_EXT
#define AL_FORMAT_VORBIS_EXT 0x10003
#endif

typedef struct {
	ALfloat x, y;
} alVector;

static ALCcontext* al_context = NULL;
static ALCdevice* al_device = NULL;

static SDL_Thread* music_player = NULL;

int sound_init(void) {
	// Open the default device.
	al_device = alcOpenDevice(NULL);
	if(al_device == NULL) {
		WARN("Unable to open default sound device");
		return -1;
	}

	// Create the OpenAL context.
	al_context = alcCreateContext(al_device, NULL);
	if(al_context == NULL) {
		WARN("Unable to create OpenAL context");
		return -2;
	}

	// We use the vorbis extension for AL to load it easily.
	if(alIsExtensionPresent("AL_EXT_vorbis") == AL_FALSE) {
		WARN("AL_EXT_vorbis not available in OpenAL context!");
		return -3;
	}

	// Set active context.
	if(alcMakeContextCurrent(al_context)==AL_FALSE) {
		WARN("Failure to set default context");
		return -4;
	}

	music_init();
	music_player = SDL_CreateThread(music_thread, NULL);

	return 0;
}


// Clean up after the sound system.
void sound_exit(void) {
	music_stop();
	music_kill();
	SDL_WaitThread(music_player, NULL);
	music_exit();

	if(al_context) {
		alcMakeContextCurrent(NULL);
		alcDestroyContext(al_context);
	}
	if(al_device) alcCloseDevice(al_device);
}

// Loads a vorbis file.
ALuint sound_sndCreate(char* filename) {
	void* ovdata;
	unsigned int size;
	ALenum err;
	ALuint buf;

	// Get the file data buffer from the packfile.
	ovdata = pack_readfile(DATA, filename, &size);

	// Bind to OpenAL buffer.
	alGenBuffers(1, &buf);
	alBufferData(buf, AL_FORMAT_VORBIS_EXT, ovdata, size, 44800);

	// Errors?
	if((err = alGetError()) != AL_NO_ERROR) {
		WARN("OpenAL erro '%d' loading sound '%s'.", err, filename);
		return 0;
	}

	// Finish up.
	free(ovdata);
	return buf;
}

void sound_sndFree(const ALuint snd) {
	alDeleteBuffers(1, &snd);
}

